package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.FlowLayout;




import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;


import javax.swing.JOptionPane;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.AboutPopUp;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.EditAlgorithmsPopUp;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.EditCollectorsPopUp;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.NetworkTreeWindow;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.SettingsPopUp;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.SimulationConfigurator;

/**
 * MenuBar for the MainFrame, which contains the different items like options
 *
 * @author Andreas T. Meyer-Berg
 */
public class MenuBar extends JMenuBar {

	/**
	 * Controller to manipulate the model
	 */
	private Controller controller;
	
	/**
	 * JMenu for the Simulation
	 */
	private JMenu mnSimulation;
	
	/**
	 * JMenu for Editing of Devices, Options etc. 
	 */
	private JMenu mnEdit;
	
	/**
	 * JMenu for help with the program
	 */
	private JMenu mnHelp;
	
	/**
	 * Serial Version
	 */
	private static final long serialVersionUID = 4293499386792032777L;
	
	/**
	 * Initialize the Menu Bar, add all the items and add the different actions
	 * @param controller Controller
	 */
	public MenuBar(Controller controller) {
		this.controller = controller;
		
		this.setLayout(new FlowLayout(FlowLayout.LEADING));

		initializeSimulationMenu();
		this.add(mnSimulation);
		initializeEditMenu();
		this.add(mnEdit);
		initializeHelpMenu();
		this.add(mnHelp);
	}
	
	/**
	 * Initializes the Simulation Menu
	 */
	private void initializeSimulationMenu(){
		mnSimulation = new JMenu("Simulation");
		JMenuItem mntmConfigureSim = new JMenuItem("Configure Sim");
		mntmConfigureSim.addActionListener(a->{
			SimulationConfigurator sim = new SimulationConfigurator(controller);
			sim.setLocationRelativeTo(this.getParent());
			sim.setVisible(true);
		});
		mnSimulation.add(mntmConfigureSim);		
	}
	
	/**
	 * Initializes the Edit Menu
	 */
	private void initializeEditMenu() {
		mnEdit = new JMenu("Edit");
		
		JMenuItem mntmDeleteModel = new JMenuItem("Delete Network");
		mntmDeleteModel.addActionListener(a -> {
			int dialogResult = JOptionPane.showConfirmDialog(this.getParent(), "Do you really want do delete all Devices, Ports, Links & Connections?");
			if(dialogResult == JOptionPane.YES_OPTION){
			  controller.getNetworkController().deleteNetworkModel();
			}
		});
		mnEdit.add(mntmDeleteModel);
		
		JMenuItem mntmOption = new JMenuItem("Settings");
		mntmOption.addActionListener(a -> {
			SettingsPopUp settings = new SettingsPopUp(controller);
			settings.setLocationRelativeTo(this.getParent());
			settings.setVisible(true);
		});
		mnEdit.add(mntmOption);
		
		JMenuItem mntmTreeView = new JMenuItem("Tree View");
		mntmTreeView.addActionListener(a -> {
			NetworkTreeWindow net = new NetworkTreeWindow(controller, this.getParent());
			net.setVisible(true);
		});
		mnEdit.add(mntmTreeView);
		
		JMenuItem mntmManageAlgos = new JMenuItem("Manage Algorithms");
		mntmManageAlgos.addActionListener(a -> {
			new EditAlgorithmsPopUp(controller, this.getParent());
		});
		mnEdit.add(mntmManageAlgos);
		
		JMenuItem mntmManageCollectors = new JMenuItem("Manage Collectors");
		mntmManageCollectors.addActionListener(a -> {
			new EditCollectorsPopUp(controller, this.getParent());
		});
		mnEdit.add(mntmManageCollectors);
	}
	
	/**
	 * Initializes the Help Menu
	 */
	private void initializeHelpMenu() {
		mnHelp = new JMenu("Help");
		JMenuItem mntmAbout = new JMenuItem("About");
		mntmAbout.addActionListener(a -> {
			AboutPopUp about = new AboutPopUp();
			about.setLocationRelativeTo(this.getParent());
			about.setVisible(true);
		});
		mnHelp.add(mntmAbout);
	}
}
